@@ -1,4 +1,5 @@
 import sys
+from collections import defaultdict
 from collections.abc import Sequence
 from copy import deepcopy
 from typing import Callable, Optional
@@ -7,11 +8,13 @@ import numpy as np
 import structlog
 import torch
 from torch.optim import Optimizer
-from torch.utils.data import DataLoader, Dataset
+from torch.utils.data import DataLoader
 from torch.utils.data.dataloader import default_collate
 from tqdm import tqdm
 
+from baal.metrics.mixin import MetricMixin
 from baal.utils.array_utils import stack_in_memory
+from baal.active.dataset.base import Dataset
 from baal.utils.cuda_utils import to_cuda
 from baal.utils.iterutils import map_on_tensor
 from baal.utils.metrics import Loss
@@ -27,7 +30,7 @@ def _stack_preds(out):
     return out
 
 
-class ModelWrapper:
+class ModelWrapper(MetricMixin):
     """
     Wrapper created to ease the training/testing/loading.
 
@@ -41,48 +44,10 @@ class ModelWrapper:
         self.model = model
         self.criterion = criterion
         self.metrics = dict()
+        self.active_learning_metrics = defaultdict(dict)
         self.add_metric("loss", lambda: Loss())
         self.replicate_in_memory = replicate_in_memory
-
-    def add_metric(self, name: str, initializer: Callable):
-        """
-        Add a baal.utils.metric.Metric to the Model.
-
-        Args:
-            name (str): name of the metric.
-            initializer (Callable): lambda to initialize a new instance of a
-                                    baal.utils.metrics.Metric object.
-        """
-        self.metrics["test_" + name] = initializer()
-        self.metrics["train_" + name] = initializer()
-
-    def _reset_metrics(self, filter=""):
-        """
-        Reset all Metrics according to a filter.
-
-        Args:
-            filter (str): Only keep the metric if `filter` in the name.
-        """
-        for k, v in self.metrics.items():
-            if filter in k:
-                v.reset()
-
-    def _update_metrics(self, out, target, loss, filter=""):
-        """
-        Update all metrics.
-
-        Args:
-            out (Tensor): Prediction.
-            target (Tensor): Ground truth.
-            loss (Tensor): Loss from the criterion.
-            filter (str): Only update metrics according to this filter.
-        """
-        for k, v in self.metrics.items():
-            if filter in k:
-                if "loss" in k:
-                    v.update(loss)
-                else:
-                    v.update(out, target)
+        self._active_dataset_size = -1
 
     def train_on_dataset(
         self,
@@ -111,20 +76,23 @@ class ModelWrapper:
         Returns:
             The training history.
         """
+        dataset_size = len(dataset)
         self.train()
+        self.set_dataset_size(dataset_size)
         history = []
-        log.info("Starting training", epoch=epoch, dataset=len(dataset))
+        log.info("Starting training", epoch=epoch, dataset=dataset_size)
         collate_fn = collate_fn or default_collate
         for _ in range(epoch):
             self._reset_metrics("train")
-            for data, target in DataLoader(
+            for data, target, *_ in DataLoader(
                 dataset, batch_size, True, num_workers=workers, collate_fn=collate_fn
             ):
                 _ = self.train_on_batch(data, target, optimizer, use_cuda, regularizer)
-            history.append(self.metrics["train_loss"].value)
+            history.append(self.get_metrics("train")["train_loss"])
 
         optimizer.zero_grad()  # Assert that the gradient is flushed.
-        log.info("Training complete", train_loss=self.metrics["train_loss"].value)
+        log.info("Training complete", train_loss=self.get_metrics("train")["train_loss"])
+        self.active_step(dataset_size, self.get_metrics("train"))
         return history
 
     def test_on_dataset(
@@ -155,15 +123,16 @@ class ModelWrapper:
         log.info("Starting evaluating", dataset=len(dataset))
         self._reset_metrics("test")
 
-        for data, target in DataLoader(
+        for data, target, *_ in DataLoader(
             dataset, batch_size, False, num_workers=workers, collate_fn=collate_fn
         ):
             _ = self.test_on_batch(
                 data, target, cuda=use_cuda, average_predictions=average_predictions
             )
 
-        log.info("Evaluation complete", test_loss=self.metrics["test_loss"].value)
-        return self.metrics["test_loss"].value
+        log.info("Evaluation complete", test_loss=self.get_metrics("test")["test_loss"])
+        self.active_step(None, self.get_metrics("test"))
+        return self.get_metrics("test")["test_loss"]
 
     def train_and_test_on_datasets(
         self,
@@ -210,7 +179,7 @@ class ModelWrapper:
                 train_dataset, optimizer, batch_size, 1, use_cuda, workers, collate_fn, regularizer
             )
             te_loss = self.test_on_dataset(test_dataset, batch_size, use_cuda, workers, collate_fn)
-            hist.append({k: v.value for k, v in self.metrics.items()})
+            hist.append(self.get_metrics())
             if te_loss < best_loss:
                 best_epoch = e
                 best_loss = te_loss
@@ -265,7 +234,7 @@ class ModelWrapper:
         loader = DataLoader(dataset, batch_size, False, num_workers=workers, collate_fn=collate_fn)
         if verbose:
             loader = tqdm(loader, total=len(loader), file=sys.stdout)
-        for idx, (data, _) in enumerate(loader):
+        for idx, (data, *_) in enumerate(loader):
 
             pred = self.predict_on_batch(data, iterations, use_cuda)
             pred = map_on_tensor(lambda x: x.detach(), pred)
@@ -466,6 +435,15 @@ class ModelWrapper:
 
         self.model.apply(reset)
 
+    def set_dataset_size(self, dataset_size: int):
+        """
+        Set state for dataset size. Useful for tracking.
+
+        Args:
+            dataset_size: Dataset state
+        """
+        self._active_dataset_size = dataset_size
+
 
 def mc_inference(model, data, iterations, replicate_in_memory):
     if replicate_in_memory:
